@@ -1,6 +1,5 @@
 __author__ = "Johannes Köster"
-__contributors__ = ["Per Unneberg"]
-__copyright__ = "Copyright 2015, Johannes Köster"
+__copyright__ = "Copyright 2015-2019, Johannes Köster"
 __email__ = "koester@jimmy.harvard.edu"
 __license__ = "MIT"
 
@@ -202,7 +201,7 @@ def makedirs(dirnames):
 def report(
     text,
     path,
-    stylesheet=os.path.join(os.path.dirname(__file__), "report.css"),
+    stylesheet=None,
     defaultenc="utf8",
     template=None,
     metadata=None,
@@ -248,6 +247,8 @@ def report(
         metadata (str):     E.g. an optional author name or email address.
 
     """
+    if stylesheeet is None:
+        os.path.join(os.path.dirname(__file__), "report.css")
     try:
         import snakemake.report
     except ImportError:
