@@ -1,6 +1,5 @@
 __authors__ = ["Tobias Marschall", "Marcel Martin", "Johannes Köster"]
-__contributors__ = ["Soohyun Lee"]
-__copyright__ = "Copyright 2015, Johannes Köster"
+__copyright__ = "Copyright 2015-2019, Johannes Köster"
 __email__ = "koester@jimmy.harvard.edu"
 __license__ = "MIT"
 
@@ -82,12 +81,15 @@ def run(
     check_md5=True,
     cores=3,
     set_pythonpath=True,
+    cleanup=True,
     **params
 ):
     """
     Test the Snakefile in path.
     There must be a Snakefile in the path and a subdirectory named
-    expected-results.
+    expected-results. If cleanup is False, we return the temporary
+    directory to the calling test for inspection, and the test should
+    clean it up.
     """
     if set_pythonpath:
         # Enforce current workdir (the snakemake source dir) to also be in PYTHONPATH
@@ -102,66 +104,91 @@ def run(
     assert os.path.exists(results_dir) and os.path.isdir(
         results_dir
     ), "{} does not exist".format(results_dir)
-    with tempfile.TemporaryDirectory(prefix="snakemake-") as tmpdir:
-        config = {}
-        # handle subworkflow
-        if subpath is not None:
-            # set up a working directory for the subworkflow and pass it in `config`
-            # for now, only one subworkflow is supported
-            assert os.path.exists(subpath) and os.path.isdir(
-                subpath
-            ), "{} does not exist".format(subpath)
-            subworkdir = os.path.join(tmpdir, "subworkdir")
-            os.mkdir(subworkdir)
-            # copy files
-            for f in os.listdir(subpath):
-                copy(os.path.join(subpath, f), subworkdir)
-            config["subworkdir"] = subworkdir
+
+    # If we need to further check results, we won't cleanup tmpdir
+    tmpdir = next(tempfile._get_candidate_names())
+    tmpdir = os.path.join(tempfile.gettempdir(), "snakemake-%s" % tmpdir)
+    os.mkdir(tmpdir)
+
+    config = {}
+
+    # handle subworkflow
+    if subpath is not None:
+        # set up a working directory for the subworkflow and pass it in `config`
+        # for now, only one subworkflow is supported
+        assert os.path.exists(subpath) and os.path.isdir(
+            subpath
+        ), "{} does not exist".format(subpath)
+        subworkdir = os.path.join(tmpdir, "subworkdir")
+        os.mkdir(subworkdir)
 
         # copy files
-        for f in os.listdir(path):
-            print(f)
-            copy(os.path.join(path, f), tmpdir)
+        for f in os.listdir(subpath):
+            copy(os.path.join(subpath, f), subworkdir)
+        config["subworkdir"] = subworkdir
 
-        # run snakemake
-        success = snakemake(
-            snakefile,
-            cores=cores,
-            workdir=path if no_tmpdir else tmpdir,
-            stats="stats.txt",
-            config=config,
-            **params
-        )
-        if shouldfail:
-            assert not success, "expected error on execution"
-        else:
-            assert success, "expected successful execution"
-            for resultfile in os.listdir(results_dir):
-                if resultfile == ".gitignore" or not os.path.isfile(
-                    os.path.join(results_dir, resultfile)
-                ):
-                    # this means tests cannot use directories as output files
-                    continue
-                targetfile = join(tmpdir, resultfile)
-                expectedfile = join(results_dir, resultfile)
-                assert os.path.exists(
-                    targetfile
-                ), 'expected file "{}" not produced'.format(resultfile)
-                if check_md5:
-                    # if md5sum(targetfile) != md5sum(expectedfile):
-                    #     import pdb; pdb.set_trace()
-                    if md5sum(targetfile) != md5sum(expectedfile):
-                        with open(targetfile) as target:
-                            content = target.read()
-                        assert False, 'wrong result produced for file "{}":\n{}'.format(
-                            resultfile, content
-                        )
+    # copy files
+    for f in os.listdir(path):
+        print(f)
+        copy(os.path.join(path, f), tmpdir)
+
+    # run snakemake
+    success = snakemake(
+        snakefile,
+        cores=cores,
+        workdir=path if no_tmpdir else tmpdir,
+        stats="stats.txt",
+        config=config,
+        verbose=True,
+        **params
+    )
+    if shouldfail:
+        assert not success, "expected error on execution"
+    else:
+        assert success, "expected successful execution"
+        for resultfile in os.listdir(results_dir):
+            if resultfile in [".gitignore", ".gitkeep"] or not os.path.isfile(
+                os.path.join(results_dir, resultfile)
+            ):
+                # this means tests cannot use directories as output files
+                continue
+            targetfile = join(tmpdir, resultfile)
+            expectedfile = join(results_dir, resultfile)
+            assert os.path.exists(targetfile), 'expected file "{}" not produced'.format(
+                resultfile
+            )
+            if check_md5:
+                # if md5sum(targetfile) != md5sum(expectedfile):
+                #     import pdb; pdb.set_trace()
+                if md5sum(targetfile) != md5sum(expectedfile):
+                    with open(targetfile) as target:
+                        content = target.read()
+                    assert False, 'wrong result produced for file "{}":\n{}'.format(
+                        resultfile, content
+                    )
+
+    if not cleanup:
+        return tmpdir
+    shutil.rmtree(tmpdir)
 
 
 def test_delete_all_output():
     run(dpath("test_delete_all_output"))
 
 
+def test_github_issue_14():
+    """Add cleanup_scripts argument to allow the user to keep scripts"""
+    # Return temporary directory for inspection - we should keep scripts here
+    tmpdir = run(dpath("test_github_issue_14"), cleanup=False, cleanup_scripts=False)
+    assert os.listdir(os.path.join(tmpdir, ".snakemake", "scripts"))
+    shutil.rmtree(tmpdir)
+
+    # And not here
+    tmpdir = run(dpath("test_github_issue_14"), cleanup=False)
+    assert not os.listdir(os.path.join(tmpdir, ".snakemake", "scripts"))
+    shutil.rmtree(tmpdir)
+
+
 def test_issue956():
     run(dpath("test_issue956"))
 
@@ -496,7 +523,6 @@ def test_spaces_in_fnames():
         dpath("test_spaces_in_fnames"),
         # cluster="./qsub",
         targets=["test bam file realigned.bam"],
-        verbose=True,
         printshellcmds=True,
     )
 
@@ -659,7 +685,6 @@ def test_default_remote():
         cores=1,
         default_remote_provider="S3Mocked",
         default_remote_prefix="test-remote-bucket",
-        verbose=True,
     )
 
 
@@ -716,12 +741,7 @@ def test_singularity_invalid():
 
 @connected
 def test_singularity_conda():
-    run(
-        dpath("test_singularity_conda"),
-        use_singularity=True,
-        use_conda=True,
-        verbose=True,
-    )
+    run(dpath("test_singularity_conda"), use_singularity=True, use_conda=True)
 
 
 def test_issue612():
@@ -957,7 +977,7 @@ def test_issue1092():
 
 
 def test_issue1093():
-    run(dpath("test_issue1093"), use_conda=True, verbose=True)
+    run(dpath("test_issue1093"), use_conda=True)
 
 
 def test_issue958():
@@ -980,16 +1000,13 @@ def test_pipes2():
     run(dpath("test_pipes2"))
 
 
-@pytest.mark.skip(
-    reason="need free AWS tier credentials and tibanna as a conda package first"
-)
 def test_tibanna():
     workdir = dpath("test_tibanna")
     subprocess.check_call(["python", "cleanup.py"], cwd=workdir)
     run(
         workdir,
         use_conda=True,
-        configfiles=["config.json"],
+        configfiles=[os.path.join(workdir, "config.json")],
         default_remote_prefix="snakemake-tibanna-test/1",
         tibanna_sfn="tibanna_unicorn_johannes",
     )
@@ -1004,7 +1021,6 @@ def test_default_resources():
 
     run(
         dpath("test_default_resources"),
-        verbose=True,
         default_resources=DefaultResources(
             ["mem_mb=max(2*input.size, 1000)", "disk_mb=max(2*input.size, 1000)"]
         ),
@@ -1052,3 +1068,34 @@ def test_batch_fail():
     from snakemake.dag import Batch
 
     run(dpath("test_batch"), batch=Batch("aggregate", 2, 2), shouldfail=True)
+
+
+def test_github_issue52():
+    run(dpath("test_github_issue52"), shouldfail=True)
+    run(dpath("test_github_issue52"), snakefile="other.smk", shouldfail=True)
+
+
+def test_github_issue78():
+    run(dpath("test_github_issue78"), use_singularity=True)
+
+
+def test_github_issue105():
+    run(dpath("test_github_issue105"))
+
+
+def test_output_file_cache():
+    test_path = dpath("test_output_file_cache")
+    os.environ["SNAKEMAKE_OUTPUT_CACHE"] = os.path.join(test_path, "cache")
+    run(test_path, cache=["a", "b", "c"])
+    run(test_path, cache=["invalid_multi"], targets="invalid1.txt", shouldfail=True)
+
+
+def test_output_file_cache_remote():
+    test_path = dpath("test_output_file_cache_remote")
+    os.environ["SNAKEMAKE_OUTPUT_CACHE"] = "cache"
+    run(
+        test_path,
+        cache=["a", "b", "c"],
+        default_remote_provider="S3Mocked",
+        default_remote_prefix="test-remote-bucket",
+    )
